/*=============================================================================
	Copyright (c) 1999-2003 pmG Worldwide, LLC.

	File:	messiah_tool_skeleton.h
	Author:	Lyle Milton
	Date:	05/01/03

		last modified: 04/24/04 - LM

			fxSkeletonEffectExists
			fxBoneTargetWeightScan
			fxBoneTargetWeightGet

	-----------------------------------------------------------------
	BONE WEIGHT DATA
	-----------------------------------------------------------------
	messiah has a simple method for extracting bone->target weight
	data that can be used to export bone weighting via plugins. There
	are 2 methods to get this data to allow for maximum flexibility.

	The first method is direct point data extraction.  If you know the
	specific bone and target point you need, this method is very direct.

	EX1: Getting weight data for a specific target point.

		FXfloat weight=0.0f;

		if (fxSkeletonEffectExists(skeletonID, 0))
		{
			if (fxBoneTargetWeightGet(boneID, targetID, pointID, &weight, 0))
			{
				//do something with the weight value
			}
		}


	In the above example, fxBoneTargetWeightGet will return the bone 
	weight data for the specified point (pointID) on the specified
	target (targetID).

	Take note of the fxSkeletonEffectExists call. This function
	determines if there is a valid BoneDeform effect attatched to the
	provided skeleton.  If there is no BoneDeform effect attatched, then
	there will be no weight data.  You're not required to perform this
	check since the fxBoneTargetWeightGet will always return a weight
	of 0.0 if there is no weight data.  However, it's good form to
	perform the check for efficiency.

	-----------------------------------------------------------------

	While the fxBoneTargetWeightGet will return the weight data for a
	specific target point, it's not very efficient if you need to get
	weight data for an entire target.  It's also not very common to
	have the pointID in advance.  In fact, it's more common to need
	to get the weight data for every point affected by a particular
	bone.
	
	You can accomplish this by scanning the bone/target for weight
	data. You first need to define your scan function, like so:

	EX2: Defining a scan function

		MyBWScanFunc(FXtool boneID, FXobject targetID, FXint pointID, FXfloat weight, MyBWScanData *data)

		OR

		FX_BWSCANFUNC(MyBWScanFunc, MyBWScanData, data)

		{
			//do something with the weight data

			return FX_TRUE;
		}


	Then you can perform your scan:

	EX3: Scanning a Bone/Target for weight data

		MyBWScanData  data;

		//initialize your data...

		if (fxSkeletonEffectExists(skeletonID, 0))
		{
			fxBoneTargetWeightScan(boneID, targetID, &data, &MyBWScanFunc, 0);
		}


	When messiah calls your function during the scan, you'll receive the
	pointID and the weight for that point.  It's important to note that
	a bone may not have a weight value for every point in a target.  In
	those cases, the point is skipped and your function is not called.
	This means that if you're filling in an array of data, be sure to
	initialize if before performing the scan.

	-----------------------------------------------------------------

	In all of the above examples, it is assumed that you know the bone
	that you'd like to scan.  There may be cases where you'll need to 
	scan all the bones that exist in a skeleton.  You can use the fxSubtool
	funcs in messiah_object.h to traverse a skeleton's bones, and at
	each bone you'd perform a scan.
	
	However, the fxBoneTargetWeightScan function can traverse all of the
	bones for you.

	EX4: Scanning a Skeleton for weight data

		MyBWScanData  data;

		//initialize your data...

		if (fxSkeletonEffectExists(skeletonID, 0))
		{
			fxBoneTargetWeightScan(skeletonID, targetID, &data, &MyBWScanFunc, 0);
		}


	Notice that the only difference between the code in EX3 & EX4 is we're
	passing a skeleton to the fxBoneTargetWeightScan function.  In this
	case, it is assumed that you want to traverse the entire skeleton
	during the scan.


	-----------------------------------------------------------------

	IMPORTANT NOTE: At the time of this API version, only Bone_Deform's
	weight data will be returned, if it exists.  Any other effect that
	uses the Skeleton tool will not return any weight data.

	Also, weight data may not exists or may not be accurate if you
	attempt to get the weights while in Setup.  So, if you're writing a
	module with the intent of extacting this data for gaming or other
	external use, you should write your module as a Customize module.
	This will ensure that messiah is not in Setup when the users access
	your plugin.

=============================================================================*/

#ifndef __MESSIAH_SKELETON
#define __MESSIAH_SKELETON

#ifndef __MESSIAH_HIDE_TYPES

	//fxSkeletonOptions,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_SFLAGS_REALTIME_ADD		(1<<0) //realtime bone drawing
	#define FX_SFLAGS_REALTIME_MODIFY	(1<<1) //realtime bone dragging & resizing
	#define FX_SFLAGS_REALTIME_SPLIT	(1<<2) //click on bone "center line" to split
	#define FX_SFLAGS_ADDBONE_MUSCLE	(1<<3) //add new bones as muscles
	#define FX_SFLAGS_ADDBONE_TARGET	(1<<4) //add new bone as target of the current muscle
	#define FX_SFLAGS_AUTODRAW_WEIGHTS	(1<<5) //automatically draw the weights on the target when selecting bones

	//fxBoneType,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_BTYPE_BONE			0
	#define	FX_BTYPE_MUSCLE			1

	//fxBoneType & fxBoneOptions,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_BFLAGS_INVERT_WEIGHTS	(1<<4)

	//fxBoneOptions,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_BFLAGS_NOSQUASH			(1<<5) //muscle doesn't fatten
	#define FX_BFLAGS_NOSTRETCH			(1<<6) //muscle doesn't get "skinny"
	#define FX_BFLAGS_NOSCALEX			(1<<7) //muscle doesn't scale on x
	#define FX_BFLAGS_NOSCALEY			(1<<8) //muscle doesn't scale on y
	#define FX_BFLAGS_AUTOLENGTH		(1<<9) //muscle doesn't function in Setup, only changes the length

	//BWScanFunc callback macro definition for use with fxBoneTargetWeightScan,,,,,,,,,
	#define FX_BWSCANFUNC(f, dt, data)		FXintf f(FXtool boneID, FXobject targetID, FXint pointID, FXfloat weight, dt *data)

#endif //__MESSIAH_HIDE_TYPES



FX_API(FXint, fxSkeletonOptionsSet)(FXtool skeletonID, FXint flags);

FX_API(FXint, fxSkeletonOptionsClear)(FXtool skeletonID, FXint flags);

FX_API(FXint, fxSkeletonOptions)(FXtool skeletonID, FXint *flags);

FX_API(FXeffect, fxSkeletonEffectExists)(FXtool skeletonID, FXint flags);

FX_API(FXint, fxBoneLengthSet)(FXtool boneID, FXdouble v);

FX_API(FXint, fxBoneRangeSet)(FXtool boneID, FXdouble v);

FX_API(FXint, fxBoneSlipSet)(FXtool boneID, FXdouble v);

FX_API(FXint, fxBoneLFactorSet)(FXtool boneID, FXdouble v);

FX_API(FXint, fxBoneFalloffSet)(FXtool boneID, FXdouble v);

FX_API(FXint, fxBoneWeightSet)(FXtool boneID, FXtool weightID, FXint invert);

FX_API(FXint, fxBoneTypeSet)(FXtool boneID, FXint type, FXobject targetID);

FX_API(FXint, fxBoneOptionsSet)(FXtool boneID, FXint flags);

FX_API(FXint, fxBoneOptionsClear)(FXtool boneID, FXint flags);

FX_API(FXint, fxBoneLength)(FXtool boneID, FXdouble *v);

FX_API(FXint, fxBoneRange)(FXtool boneID, FXdouble *v);

FX_API(FXint, fxBoneSlip)(FXtool boneID, FXdouble *v);

FX_API(FXint, fxBoneLFactor)(FXtool boneID, FXdouble *v);

FX_API(FXint, fxBoneFalloff)(FXtool boneID, FXdouble *v);

FX_API(FXint, fxBoneWeight)(FXtool boneID, FXtool *weightID);

FX_API(FXint, fxBoneType)(FXtool boneID, FXint *type, FXobject *targetID);

FX_API(FXint, fxBoneOptions)(FXtool boneID, FXint *flags);

FX_API(FXint, fxBoneTargetWeightScan)
(
	FXtool		boneID,
	FXobject	targetID,
	FXvoid		*data,
	FXint		*BWScanFunc(FXtool boneID, FXobject targetID, FXint pointID, FXfloat weight, FXvoid *data),
	FXint		flags
);

FX_API(FXint, fxBoneTargetWeightGet)(FXtool boneID, FXobject targetID, FXint pointID, FXfloat *weight, FXint flags);




#define _MESSIAH_PTR_SKELETON	messiah_apicomponent = FX_APIFUNCS_TOOLTYPE;\
								FX_PTR(fxSkeletonOptionsSet);\
								FX_PTR(fxSkeletonOptionsClear);\
								FX_PTR(fxSkeletonOptions);\
								FX_PTR(fxSkeletonEffectExists);\
								FX_PTR(fxBoneLength);\
								FX_PTR(fxBoneRange);\
								FX_PTR(fxBoneSlip);\
								FX_PTR(fxBoneLFactor);\
								FX_PTR(fxBoneFalloff);\
								FX_PTR(fxBoneWeight);\
								FX_PTR(fxBoneType);\
								FX_PTR(fxBoneLengthSet);\
								FX_PTR(fxBoneRangeSet);\
								FX_PTR(fxBoneSlipSet);\
								FX_PTR(fxBoneLFactorSet);\
								FX_PTR(fxBoneFalloffSet);\
								FX_PTR(fxBoneWeightSet);\
								FX_PTR(fxBoneTypeSet);\
								FX_PTR(fxBoneTargetWeightScan);\
								FX_PTR(fxBoneTargetWeightGet);\
								FX_PTR(fxBoneOptions);\
								FX_PTR(fxBoneOptionsSet);\
								FX_PTR(fxBoneOptionsClear)


#endif //__MESSIAH_SKELETON
